// borrowed from GlobalX.Web CdmHelpers implementation
// supports V1 CDM for the purpose of webpage integration
function CdmHelpers(AppSettings) {
	'ngInject';

	const service = {
		getDatumVal: getDatumVal,
		parseDataModel: parseDataModel
	};

	function getDatumVal(datumList, key) {
	    for (var j = 0; j < datumList.length; j++) {
	        var datum = datumList[j];
	        if (datum.Key == key) {
	            return datum.Value;
	        }
	    }
	    return null;
	}

	function parseDataModel(modelTypes, relationshipType, predicate, model) {
        var found = [];
        modelTypes = asArray(modelTypes);
        for (var i = 0; i < modelTypes.length; i++)
            parseDataModelForTypes(modelTypes[i], relationshipType, predicate || function () { return true; }, model, null, found, 0);
        return found;
    }

	function parseDataModelForTypes(modelType, relationshipType, predicate, model, parent, found, depth) {
        if (depth++ > AppSettings.dataModels.maxParseDepth || !model) return;

        if (modelType && model.$type) {
            var t = model.$type.replace(/,.*$/g, '');
            if (modelType == t && predicate(model, parent)) found.push(model);
        }

        if (model.RelatedEntities) {
            for (var i = 0; i < model.RelatedEntities.length; i++) {
                var r = model.RelatedEntities[i];
                if (!r || !r.Entity) continue;
                if (relationshipType && (r.RelationshipType != relationshipType && relationshipType != 'NA')) continue;
                var e = r.Entity;
                parseDataModelForTypes(modelType, relationshipType, predicate, e, model, found, depth);
            }
        }

        if (model.Relationships) {
            for (var i = 0; i < model.Relationships.length; i++) {
                var r = model.Relationships[i];
                if (!r || !r.Entity) continue;
                if (relationshipType && (r.RelationshipType != relationshipType &&  relationshipType != 'NA')) continue;
                var e = r.Entity;
                parseDataModelForTypes(modelType, relationshipType, predicate, e, model, found, depth);
            }
        }
        
        for (var k in model) {
            if (model.hasOwnProperty(k) && k != AppSettings.dataModels.relationships && k != AppSettings.dataModels.relatedEntities && typeof model[k] == 'object') {
                var modelsToCheck = asArray(model[k]);
                for (var i = 0; i < modelsToCheck.length; i++)
                    parseDataModelForTypes(modelType, relationshipType, predicate, modelsToCheck[i], model, found, depth);
            }
        }
    }

    function asArray(obj) {
        return obj instanceof Array ? obj : [obj];
    }

  	return service;
}

export default {
  name: 'CdmHelpers',
  fn: CdmHelpers
};