describe('Unit: CDM Helpers', function() {

  let subject, appSettingsMock;

  beforeEach(function() {
    angular.mock.module('app');

    appSettingsMock = {
      dataModels: {
        maxParseDepth: 10,
        relationships: 'Relationships',
        relatedEntities: 'RelatedEntities'
      }
    };

    angular.mock.module(($provide) => {
      $provide.value('AppSettings', appSettingsMock);
    });
    
    angular.mock.inject((CdmHelpers) => {
      subject = CdmHelpers;
    });
  });

  it('should exist', function() {
    expect(subject).toBeDefined();
  });

  it('should get a datum value from a list', function() {
    var datumList = [
      { Key: 'no match 1', Value: 'no match val 1' },
      { Key: 'no match 2', Value: 'no match val 2' },
      { Key: 'no match 3', Value: 'no match val 3' },
      { Key: 'match', Value: 'match val' },
      { Key: 'no match 4', Value: 'no match val 4' }
    ];

    var key = 'match';

    var result = subject.getDatumVal(datumList, key);

    expect(result).toBe('match val');
  });

  it('should not get a datum value from a list when no match', function() {
    var datumList = [
      { Key: 'no match 1', Value: 'no match val 1' },
      { Key: 'no match 2', Value: 'no match val 2' },
      { Key: 'no match 3', Value: 'no match val 3' },
      { Key: 'no match 4', Value: 'no match val 4' }
    ];

    var key = 'match';

    var result = subject.getDatumVal(datumList, key);

    expect(result).toBe(null);
  });

  it('should parse a valid related entity from a payload', function() {
    var payload = {
      MatterReference: 'a matter ref',
      MatterEntity: 'matter entity',
      RelatedEntities: [
        GenerateRelatedEntity('SettlementTitleReference', {
          '$type': 'GlobalX.Common.DataModel.RealProperty, GlobalX.CDM.Common',
          'TitleReference': '1/SP1'
        }),
        GenerateRelatedEntity('SettlementTitleReference', {
          '$type': 'GlobalX.Common.DataModel.RealProperty, GlobalX.CDM.Common',
          'TitleReference': '2/SP1'
        }),
        GenerateRelatedEntity('Purchaser', {
          '$type': 'GlobalX.Common.DataModel.Individual, GlobalX.CDM.Common',
          'GivenNames': 'Donald',
          'Surname': 'Trump',
          'OtherData': [{
              'Key': 'MiddleName',
              'Value': 'J'
          }]
        }),
        GenerateRelatedEntity('Purchaser', {
          '$type': 'GlobalX.Common.DataModel.Company, GlobalX.CDM.Common',
          'Name': 'GlobalX'
        }),
        GenerateRelatedEntity('Vendor', {
          '$type': 'GlobalX.Common.DataModel.Company, GlobalX.CDM.Common',
          'Name': 'Suncorp'
        })
      ]
    };

    var individualPurchasers = subject.parseDataModel('GlobalX.Common.DataModel.Individual', 'Purchaser', null, payload);
    var settlementTitleReferences = subject.parseDataModel('GlobalX.Common.DataModel.RealProperty', 'SettlementTitleReference', null, payload);
    var specificSettlementTitleReference = subject.parseDataModel('GlobalX.Common.DataModel.RealProperty', 'SettlementTitleReference', function(model) { 
      return model.TitleReference === '2/SP1';
    }, payload);
    var allPurchasers = subject.parseDataModel(['GlobalX.Common.DataModel.Individual', 'GlobalX.Common.DataModel.Company'], 'Purchaser', null, payload);
    var aVendor = subject.parseDataModel(['GlobalX.Common.DataModel.Individual', 'GlobalX.Common.DataModel.Company'], 'Vendor', null, payload);

    expect(individualPurchasers.length).toBe(1);
    expect(individualPurchasers[0].Surname).toBe('Trump');

    expect(settlementTitleReferences.length).toBe(2);
    expect(settlementTitleReferences[0].TitleReference).toBe('1/SP1');
    expect(settlementTitleReferences[1].TitleReference).toBe('2/SP1');

    expect(specificSettlementTitleReference.length).toBe(1);
    expect(specificSettlementTitleReference[0].TitleReference).toBe('2/SP1');

    expect(allPurchasers.length).toBe(2);
    expect(allPurchasers[0].Surname).toBe('Trump');
    expect(allPurchasers[1].Name).toBe('GlobalX');

    expect(aVendor.length).toBe(1);
    expect(aVendor[0].Name).toBe('Suncorp');
  });

  //helpers
  function GenerateRelatedEntity(type, entity) {
    return {
      '$type': 'GlobalX.Common.DataModel.Relationship, GlobalX.CDM.Common',
      'RelationshipType': type,
      'Entity': entity
    };
  }
});